Vue.component('maker-editor-pages', {
    template:
        '<div class="editor_pages">' +
        '<div class="editor_tips">点击图片或文字开始编辑</div>' +
        '<div class="editor_type">' +
        '<ul>' +
        '<li @click.stop="reName()"><i class="iconfont icon-tuceng"></i><span>改菜名</span></li>' +
        '<li @click.stop="reSize()" v-if="$root.type != \'recipe\'"><i class="iconfont icon-chicun"></i><span>更改尺寸</span></li>' +
        '<li @click.stop="reTemp()" v-if="/recipe|lightbox/.test($root.type)"><i class="iconfont icon-moban"></i><span>更换模板</span></li>' +
        '<li @click.stop="$root.backHistory()"><i class="iconfont icon-chexiao"></i><span>撤销编辑</span></li>' +
        '</ul>' +
        '</div>' +
        '<div class="editor_batch" v-if="batch" ref="batch">' +
        '<div class="title"></div>' +
        '<vue-scroller uniqueKey="id" :items="[]"  :variable="true"  :infinite="false"  :pulldown="false" >' +
        '<template slot="prepend" >' +
        '<ul v-for="goulp in batch"><li v-for="item in goulp.list"><label><span v-text="item.markname"></span><input type="text" v-model="item.value" @click="$setKeyboard" /></label></li></ul>' +
        '</template>' +
        '</vue-scroller>' +
        '<h6><a @click="batch=null">取消修改</a><a @click="batchRename()">确定修改</a></h6>' +
        '</div>' +
        '</div>',
    props: { page: Object, select: Object },
    data: function () {
        return {
            loading: false, // 是否正在更新中
            showsize: true,
            showpage: false,
            iscroll: null,
            batch: null,
        }
    },
    methods: {
        reName: function () {
            var page = this.page
            var batch = []
            var goulp = {}
            var name = {
                title: { name: '菜名', sort: 1 },
                price: { name: '价格', sort: 2 },
                unit: { name: '单位', sort: 3 },
                money: { name: '货币', sort: 4 },
                intro: { name: '介绍', sort: 5 },
            }

            // 按pid分组
            page.module.map(function (item) {
                if (item.type !== 'texts') {
                    return
                }
                var text = JSON.parse(JSON.stringify(item))
                var info = name[item.mark] || { name: '文字', sort: 999 }

                text.markname = info.name
                text.sort = info.sort

                goulp[item.pid] = goulp[item.pid] || []
                goulp[item.pid].push(text)
            })

            for (var key in goulp) {
                goulp[key].sort(function (a, b) {
                    return a.sort - b.sort
                })
                batch.push({ pid: key, list: goulp[key] })
            }

            if (batch.length > 0) {
                this.batch = batch.reverse()
            } else {
                this.$toast('没有可编辑的文字')
            }
        },

        // 修改尺寸
        reSize: function () {
            if (this.$root.type && this.$root.type == 'poster') {
                this.$toast('海报不可更换尺寸')
                return
            }

            var html = [
                '<label><input placeholder="宽度(cm)" type="text" name="width" value="' +
                    this.page.width +
                    '" /></label>',
                '<label><input placeholder="高度(cm)" type="text" name="height" value="' +
                    this.page.width +
                    '" /></label>',
            ]
            this.$confirm(html.join(''), ['确定', '取消'], function (text, warp) {
                if (text == '确定') {
                    var form = warp.querySelector('form')
                    var width = parseFloat(form.width.value)
                    var height = parseFloat(form.height.value)
                    if (!width || !height) {
                        this.$toast('请输入正确的宽度高度值')
                        return true
                    } else {
                        this.page.width = width
                        this.page.height = height
                        this.$emit('global:maker_rebuild')
                        this.$root.setHistory()
                        this.$toast(
                            '修改尺寸(' + this.page.width + 'x' + this.page.height + 'cm)完成'
                        )
                    }
                }
            })
        },

        // 修改模板
        reTemp: function () {
            if (this.$root.type == 'lightbox') {
                this.$toLink('lightbox_temp.html', {
                    width: this.page.width,
                    height: this.page.height,
                    lightboxId: this.page.lightboxId,
                    isSelect: true,
                })
            } else if (this.$root.type == 'recipe') {
                this.$toLink('recipe_temp.html', {
                    templateId: this.page.templateId,
                    id: this.page.recipeId,
                    pagetype: this.page.pagetype,
                    width: this.page.width,
                    height: this.page.height,
                })
            }
        },

        // 批量修性文字
        batchRename: function () {
            var page = this.page
            var batch = this.batch
            var idobj = {}

            // 按pid分组
            batch.map(function (goulp) {
                ;(goulp.list || []).map(function (item) {
                    idobj[item.id] = item
                })
            })

            page.module.map(function (item) {
                if (item.type !== 'texts') {
                    return
                }
                idobj[item.id] && (item.value = idobj[item.id].value)
            })

            this.batch = null
            this.$root.setHistory()
            this.$toast('更改文字完成')
        },

        // 获取有效的id值
        getId: function (count) {
            var id = 1
            var ids = this.page.module.map(function (i) {
                return i.id
            })
            var res = []
            var next = true
            count = count || 1

            while (next) {
                ids.indexOf(id) < 0 && res.push(id)
                id = id + 1
                next = res.length < count
            }
            return res
        },

        // 添加文字
        addTexts: function () {
            var that = this
            var item = {
                id: this.getId()[0],
                pid: 0,
                type: 'texts',
                mark: '',
                origin: 'cm',
                x: 0,
                y: 0,
                w: 40,
                h: 40,
                offsetX: 0,
                offsetY: 0,
                offsetW: 0,
                offsetH: 0,
                rotate: 0,
                transparent: 1,
                imageW: 0,
                imageH: 0,
                fontSize: 3,
                value: '请输入文字',
                shadowSize: 0,
                shadowX: 0,
                shadowY: 0,
                shadowColor: '#000',
                fontFamily: '',
                lineHeight: 1.5,
                color: '#333',
                textAlign: 'left',
                verticalAlign: 'top',
                baseLine: 'middle',
                writingMode: 'lr-tb',
                letterSpacing: 0,
                background: '',
                strokeSize: 0,
                strokeColor: '#f00',
            }

            this.page.module.push(item)
            this.$emit('global:maker_change', item)
            this.$emit('global:maker_select', item)
            this.$root.setHistory()
        },

        // 添加图片
        addImage: function () {
            var item = {
                id: this.getId()[0],
                pid: 0,
                type: 'image',
                mark: '',
                origin: 'cm',
                x: 0,
                y: 0,
                w: 30,
                h: 30,
                offsetX: 0,
                offsetY: 0,
                offsetW: 1,
                offsetH: 1,
                rotate: 0,
                transparent: 1,
                imageW: 0,
                imageH: 0,
                url: '',
                gid: '',
                rid: '',
            }

            this.page.module.push(item)
            this.$emit('global:maker_change', item)
        },

        // 更换模板
        makerRetemp: function (temp) {
            var that = this

            // 确认
            this.$confirm(
                '替换模板将会丢失当前灯箱片数据哦，您确认替换吗',
                ['确定替换', '暂不更换'],
                function (value) {
                    if (value !== '确定替换') {
                        return
                    }

                    var replace = that.$replacePage(that.page, temp)
                    that.page.temp_id = replace.temp_id
                    that.page.name = replace.names // 是	string	模板名称
                    that.page.width = replace.width // 是	int	模板宽度，在获取模板的时候已经设置为用户自定宽度
                    that.page.height = replace.height // 是	int	模板高度，在获取模板的时候已经设置为用户自定高度
                    that.page.quantity = replace.quantity // 非必填，菜品数量
                    that.page.module = replace.module
                    that.page.unit = 'cm' // 非必填，单位
                    that.page.preview = '' // 非必填，生成效果缩略图

                    that.$root.setHistory()
                    that.$toast('更换模板完成')
                }.bind(this)
            )
        },
    },

    created: function () {
        this.$on('global:maker_retemp', this.makerRetemp)

        // 判断内容有没有变更
        this.$root.setHistory()
    },
})
