Vue.use(function(Vue, options) {


	var images = {};
	var fonts = {}; // 字体缓存

	// 数据过滤器
	var dataFilter = function(value,size){
		var val = parseFloat(value) || 0;
		var res = /cm$/.test(value) ? val / size : val / 100;
		return res;
	}
	// 四舍五入浮点小数计算不准确问题
	var dataRint = function(value){return Math.round(parseFloat((value || 0).toPrecision(12)));}


	// 获取文字路径
	var makerGetTextsFonts = function(data,callback,count){

		var that = this;
		var rid = data.fontFamily;
		var value = data.texts || '';
		var txt = [];
		var family = {};

		// 取文字
		value.split('').map(function(i){
			if(!i){return;}

			if(txt.indexOf(i) >= 0){
				// 如果文字已经存在，则不管
			}else if(fonts[rid] && fonts[rid][i]){
				// 已经缓存的字体
				family[i] = fonts[rid][i];
			}else{
				txt.push(i);	
			}
		});

		if(txt.length >0){

			// Vue.prototype.$post('fonts',{texts:JSON.stringify([{text:txt.join(''),res_id:rid}])},function(json){
			Vue.prototype.$ajax('FONTPATH',{texts:JSON.stringify([{text:txt.join(''),id:rid}])},function(json){

				var list = (json || {}).data || [];
				var font = {};

				if(list.length && list.map){

					// 取数据成功
					list.map(function(item){

						if(!item){return;}
						if(!item.webfont){return;}
						if(!item.webfont.map){return;}
						fonts[rid] = fonts[rid] || {};
						item.webfont.map(function(i){
							family[i.text] = i;
							fonts[rid][i.text] = i;
						});
					});

					callback && callback(family);
				}else if(count || count < 5){
					// 加载成功则再次获取，最多获取5次
					count = count || 0;
					count = count + 1;
					setTimeout(function(){makerGetTextsFonts(data,callback,count)},2000);
				}else{
					// 出错
					callback && callback({});
				}
			});

		}else{
			callback && callback(family);
		}
	}


	// 获取svg
	var getTextsSvg = function(option, family) {

		// svg 的path定位是基于fontSize = 100 来计算的，所以其他值要相就做处理
		var texts = option.texts.split('');
		var fontSize = 100;
		var warpW = (fontSize * option.warpW) / option.fontSize;
		var warpH = (fontSize * option.warpH) / option.fontSize;
		var width = (fontSize * option.width) / option.fontSize;
		var height = (fontSize * option.height) / option.fontSize;
		var left = (fontSize * option.left) / option.fontSize;
		var top = (fontSize * option.top) / option.fontSize;
		var letterSpacing = (fontSize * option.letterSpacing) / option.fontSize;
		var strokeSize = (fontSize * option.strokeSize) / option.fontSize;
		var shadowX = (fontSize * option.shadowX) / option.fontSize;
		var shadowY = (fontSize * option.shadowY) / option.fontSize;
		var shadowSize = (fontSize * option.shadowSize) / option.fontSize;
		var lineHeight = (fontSize * option.lineHeight) / option.fontSize;

		var textAlign = option.textAlign;
		var verticalAlign = option.verticalAlign;
		var baseLine = option.baseLine;
		var writingMode = option.writingMode;
		var writeWordanti = option.writeWordanti;
		var writeLineanti = option.writeLineanti;
		var writeVertical = option.writeVertical;
		var fontWeight = option.fontWeight;
		var fontColor = option.fontColor;
		var fontImage = option.fontImage;
		var strokeColor = option.strokeColor;
		var shadowColor = option.shadowColor;

		var centerX = left + width / 2;
		var centerY = top + height / 2;
		var rotate = `rotate(${option.rotate},${centerX} ${centerY})`;

		var outerW = 2 * option.warpW;
		var outerH = 2 * option.warpH;

		// 返回的结果
		var result = {
			lines: [], // 每行
			items: {}, // svg 对象缓存
			texts: [], // 字符位置列表

			style: '', // 样式
			defs: '', // 定义路径
			symbol: '', // 重复的行排列
			display: '', // 显示区域

			width: 0, // 段落真实宽度
			height: 0, // 段落真实高度
			offsetx: 0,
			offsety: 0,
		};

		// 1.把文字形状缓存起来
		// 2.把文字一行一行的放入
		// 3.得到每行的宽高
		texts.map(function(txt, index) {
			var item = result.items[txt];
			var line = result.lines[result.lines.length - 1];
			var newline = false;

			// 缓存
			if (!item) {
				var svg = family[txt] || {};
				var path = `<path id="${txt}" vector-effect="non-scaling-stroke" d="${svg.path}" />`;
				item = {
					txt: txt,
					width: svg.width || fontSize,
					height: svg.height || fontSize,
					path: path,
				};

				result.items[txt] = item;
				result.defs += item.path;
			}

			// 如果没有最后一行，或者宽度超过了，则另起一行
			if (!line) {
				newline = true;
			} else if (writeVertical) {
				newline = height && line.height + item.height + letterSpacing > height;
			} else {
				newline = width && line.width + item.width + letterSpacing > width;
			}

			// 另起一行
			if (newline || txt == '\n') {
				line = {
					x: 0,
					y: 0,
					width: 0,
					height: 0,
					value: []
				};
				result.lines.push(line);

				if (writeVertical) {
					// 竖排
					line.width = lineHeight;
					result.width = line.width * result.lines.length;
				} else {
					// 横排
					line.height = lineHeight;
					result.height = line.height * result.lines.length;
				}
			}

			// 换行
			if (txt == '\n') {
				// 不作处理
			} else if (writeVertical) {
				// 竖排
				line.value.push({
					x: 0,
					y: 0,
					w: item.width,
					h: item.height,
					t: txt
				});
				line.height = line.height + item.height + (newline ? 0 : letterSpacing);
				line.height > result.height && (result.height = line.height);
			} else {
				// 默认横排
				line.value.push({
					x: 0,
					y: 0,
					w: item.width,
					h: item.height,
					t: txt
				});
				line.width = line.width + item.width + (newline ? 0 : letterSpacing);
				line.width > result.width && (result.width = line.width);
			}
		});

		// 书写的行方向如果是[右往左|下往上],则写入的顺序正好相反
		writeLineanti && result.lines.reverse();

		// 这一段用于计算对齐方式，相对于容器的位置
		// 文字水平对齐
		if (textAlign == 'right') {
			// 竖排
			if (writeVertical) {
				result.offsety = height - result.height;
			} else {
				result.offsetx = width - result.width;
			}
		} else if (textAlign == 'center') {
			// 竖排
			if (writeVertical) {
				result.offsety = (height - result.height) / 2;
			} else {
				result.offsetx = (width - result.width) / 2;
			}
		}

		if (verticalAlign == 'bottom') {
			/lr$/.test(writingMode) && (result.offsetx = width - result.width);
			/tb$/.test(writingMode) && (result.offsety = height - result.height);
		} else if (verticalAlign == 'middle') {
			// 竖排
			if (writeVertical) {
				result.offsetx = (width - result.width) / 2;
			} else {
				result.offsety = (height - result.height) / 2;
			}
		} else {
			/rl$/.test(writingMode) && (result.offsetx = width - result.width);
			/bt$/.test(writingMode) && (result.offsety = height - result.height);
		}

		// 对每一行文字进行处理
		result.lines.map(function(line, index) {
			var x = 0;
			var y = 0;
			var id = 'line' + index;
			var base = baseLine; // 基线对齐
			var align = textAlign; // 文本对齐 [left|center|right]
			var uses = ''; // <use xlink:href="#您" x="0" y="0" />

			// 书写方向如果是[右往左|下往上]，则倒序
			writeWordanti && line.value.reverse();

			// 书写的行方向如果是[右往左|下往上]，则baseline对齐方式正好相反
			// /(rl|bt)$/.test(writingMode) && (base = {top:'bottom',bottom:'top',middle:'middle'}[base]);

			if (writeVertical) {
				// 竖排

				line.x = index * lineHeight;
				line.y = align == 'right' ? result.height - line.height : align == 'center' ? (result.height - line.height) / 2 : 0;

				line.value.map(function(text){
					var x1 = base == 'bottom' ? line.width - text.w : base == 'middle' ? (line.width - text.w) / 2 : 0;
					var y1 = y;
					y = y + text.h + letterSpacing;

					x1 = result.offsetx + line.x + x1 + left;
					y1 = result.offsety + line.y + y1 + top;

					// x1 = result.offsetx + line.x + x1;
					// y1 = result.offsety + line.y + y1;

					result.symbol += `<use xlink:href="#${text.t}" x="${x1}" y="${y1}" width="${text.w}" height="${text.h}" />`;
				});
			} else {
				// 横排
				line.x = align == 'right' ? result.width - line.width : align == 'center' ? (result.width - line.width) / 2 : 0;
				line.y = index * lineHeight;

				line.value.map(function(text){
					var x1 = x;
					var y1 = base == 'bottom' ? line.height - text.h : base == 'middle' ? (line.height - text.h) / 2 : 0;

					x = x + text.w + letterSpacing;
					x1 = result.offsetx + line.x + x1 + left;
					y1 = result.offsety + line.y + y1 + top;

					// x1 = result.offsetx + line.x + x1;
					// y1 = result.offsety + line.y + y1;

					result.symbol += `<use xlink:href="#${text.t}" x="${x1}" y="${y1}" width="${text.w}" height="${text.h}" />`;
				});
			}
		});

		// 描边
		if (strokeColor && strokeSize) {
			result.style += `#box{stroke: ${strokeColor};stroke-width: ${strokeSize};}`;
		}

		// 阴影
		if (shadowSize || shadowX || shadowY) {
			result.defs += '<filter id="filter" x="-200%" y="-200%" width="400%" height="400%">';
			result.defs += '<feGaussianBlur result="blurOut" in="SourceAlpha" stdDeviation="'+shadowSize+'"/>';
			result.defs += '<feOffset result="offsetblur" dx="'+shadowX+'" dy="'+shadowY+'"/>';
			result.defs += '<feFlood flood-color="'+shadowColor+'" flood-opacity="1" result="offsetColor"/>';
			result.defs += '<feComposite  in2="offsetblur" operator="in" result="blurOut"/>';
			result.defs += '<feMerge><feMergeNode/><feMergeNode in="SourceGraphic"/></feMerge>';
			result.defs += '</filter>';
			result.style += '#box{filter: url(#filter);}';
		}

		// 文字颜色或填充图片
		if (fontImage) {
			var match = (fontImage.match(/\d+[_-x]\d+/) || ['100_100'])[0].split(/[_-x]/);
			var imgWidth = parseInt(match[0]) || 100;
			var imgHeight = parseInt(match[1]) || 100;

			result.defs += '<pattern id="pattern" width="'+imgWidth+'" height="'+imgHeight+'" patternUnits="userSpaceOnUse"><image width="'+imgWidth+'" height="'+imgHeight+'" xlink:href="'+fontImage+'"/></pattern>';
			result.defs += '<mask id="mask"><use xlink:href="#symbol" x="'+result+'offsetx}" y="'+result+'offsety}" width="'+result.width+'}" height="'+result.height+'}" fill="white"/></mask>';
			result.display += '<use xlink:href="#symbol" x="0" y="0" width="'+width+'" height="'+height+'" transform="'+rotate+'" id="box" />';
			result.display += '<rect x="0" y="0" fill="url(#pattern)" width="'+width+'" height="'+height+'" transform="'+rotate+'" mask="url(#mask)"/>';
		} else {
			result.display += '<use xlink:href="#symbol" x="0" y="0" width="'+warpW+'" height="'+warpH+'" transform="'+rotate+'" id="box" />';
			result.display += '<use xlink:href="#symbol" x="0" y="0" width="'+warpW+'" height="'+warpH+'" transform="'+rotate+'" fill="'+fontColor+'"/>';
		}


		result.style = '<style>/* <![CDATA[ */ '+result.style+'}/* ]]> */</style>';
		result.defs = '<defs>'+result.defs+'}</defs>';
		result.symbol = '<symbol id="symbol" x="0" y="0" viewBox="0 0 '+warpW+' '+warpH+'">'+result.symbol+'}</symbol>';

		return '<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" width="'+warpW+'" height="'+warpH+'" viewBox="0 0 '+warpW+' '+warpH+'" >'+result.style+result.defs+result.symbol+result.display+'</svg>';


	}



	// 函数截流与去抖动，用于高频率高耗性能的渲染
	// 让一个函数无法在很短的时间间隔内连续调用，只有当上一次函数执行后过了你规定的时间间隔，才能进行下一次该函数的调用
	// 1.keyup像后台请求检验的时候，可以使用防抖函数，不然我们每按一次键盘就请求一次，请求太频繁
	// 2.resize 窗口大小调整的时候，我们可以采用防抖技术也可以使用节流；
	// 3.mousemove 鼠标移动事件我们既可以采用防抖也可以使用节流；
	// 4.scroll 滚动条触发的事件，当然既可以采用防抖也可以采用节流；
	// 5.连续高频发的事件都可以采用这两种方式去解决，优化页面性能；
	Vue.prototype.$throttle = function(fn, delay, mustRunDelay){
		var timer = null;
		var t_start;
		return function(){
			var context = this, args = arguments, t_curr = performance.now();
			clearTimeout(timer);
			if(!t_start){
				t_start = t_curr;
			}
			if(t_curr - t_start >= mustRunDelay){
				fn.apply(context, args);
				t_start = t_curr;
			}else {
				timer = setTimeout(function(){
					fn.apply(context, args);
				}, delay);
			}
		};
	}


	// 合成数据
	Vue.prototype.$buildPage = function(temp,images){

		var groups = {}; // 新模板分组
		var dishes = []; // 新模板菜品位置
		var pagename = ''; // 页面标题

		temp.module = temp.module || JSON.parse(temp.data || '[]');
		delete temp.data;

		this.$makerUpdateVersion(temp);

		// 解析新模板数据，按 pid 进行分组
		temp.module.map(function(item){
			groups[item.pid] = groups[item.pid] || {};
			groups[item.pid][item.mark] = item;
			item.mark == 'dishes' && dishes.push(item);
		});


		// 解析新模板数据，设置图片和 title
		dishes.map(function(item,idx){
			var image = images[idx] || {middleWidth:1,middleHeight:1,name:'',gId:'',middleUrl:''};
			var group = groups[item.pid] || {};

			// 修改图片 dishe
			item.url = image.middleUrl;
			item.gid = image.gId || '';
			item.rid = image.res_id || '';

			// 图片宽度、高度、偏移值
			item.title = image.name;
			item.imageW = parseInt(image.middleWidth);
			item.imageH = parseInt(image.middleHeight);

			item.offsetX = 0;
			item.offsetY = 0;
			item.offsetW = 1;
			item.offsetH = 1;

			group.title && (group.title.value = item.title); // 修改菜品名称
			// group.price && (group.price.value = item.price); // 修改菜品价格
			pagename = pagename || item.title; // 页面名称
		});

		// 这两个数据不变：更l_id，check
		temp.name = pagename;
		temp.module = temp.module;
		temp.quantity = dishes.length;
		temp.width = parseFloat(temp.width);
		temp.height = parseFloat(temp.height);
		temp.version = '2.0';
		return temp;
	}

	// 替换模板
	Vue.prototype.$replacePage = function(page,temp){

		var images = []; // 旧模板图片
		var list = page.module || JSON.parse(page.data || '[]');

		// 解析原模板数据，拿到图片与 title
		list.map(function(item){

			if(item.mark !== 'dishes'){return;}

			var title = page.module.filter(function(i){return i.mark == 'title' && i.pid == item.pid})[0];

			images.push({
				middleWidth : item.imageW,
				middleHeight : item.imageH,
				middleUrl: item.url,
				gId : item.gid || '',
				res_id : item.rid || '',
				name: item.title || (title ? title.value : ''),
			});
		});
		return this.$buildPage(temp,images);
	}




	// 加载图片
	Vue.prototype.$makerGetImageLoad = function(data,option,callback){
		if(!data.url){callback && callback(null);return;}
		var img = new Image;
		img.crossOrigin = "anonymous"; //关键 解决跨域
		img.onload = function(){callback && callback(img);};
		img.onerror = function(){callback && callback(null);};
		img.src = this.$api('IMAGE') + data.url;
	}


	// 加载视频缩略图
	Vue.prototype.$makerGetVideoLoad = function(data,option,callback){


		if(!data.url){callback && callback(null);return;}

		var image = new Image();
		var video = document.createElement('video');
		var canvas = document.createElement('canvas');


		// 如果有封面，则渲染封面
		// 如果没有封面，就用视频第一帧渲染
		if(data.poster){

			image.crossOrigin = "anonymous"; //关键 解决跨域
			image.onload = function(){callback && callback(image);};
			image.onerror = function(){callback && callback(null);};
			image.src = this.$api('IMAGE') + data.poster;
		}else{

			// video.setAttribute('crossOrigin', 'anonymous');//处理跨域
			// video.setAttribute("preload", 'preload'); // 设置为meta貌似无效    复制代码        video.src = e.target.result
			// video.setAttribute("muted", 'muted');
			// video.setAttribute("autoplay", 'autoplay');

			// video.addEventListener("error",function(){
			// 	callback && callback(null);
			// })


			// 这里有兼容问题
			// loadeddata
			// canplay
			// video.addEventListener("loadeddata",function(){
			// 	canvas.width = video.videoWidth || 0;
			// 	canvas.height = video.videoHeight || 0;
			// 	canvas.getContext("2d").drawImage(video, 0, 0, canvas.width, canvas.height); //绘制canvas
			// 	image.src = canvas.toDataURL('image/png'); //转换为base64
			// })

			canvas.width = data.imageW || 0;
			canvas.height = data.imageH || 0;
			canvas.getContext("2d").fillStyle = '#eee';
			canvas.getContext("2d").fillRect(0 ,0,canvas.width,canvas.height)
			canvas.getContext("2d").drawImage(video, 0, 0, canvas.width, canvas.height); //绘制canvas
			image.src = canvas.toDataURL('image/png'); //转换为base64

			// video.setAttribute('src', this.$api('IMAGE') + data.url);
			image.onload = function(){callback && callback(image);};
			image.onerror = function(){callback && callback(null);};

		}
		
	}


	// 取文字图片
	Vue.prototype.$makerGetTextsImage = function(data,option,callback){

		// 防止阴影与描边被剪掉，使用整个页面大小做成图，性能可优化的点
		// var diagonal = Math.sin(45) * Math.sqrt(Math.pow(option.pageW,2)+Math.pow(option.pageH,2)); // 对角线长度
		var scale = option.warpS
		var mini = Math.min(option.pageW,option.pageH); // 最小边
		var unit = 100 / data.fontSize / mini;
		var defu = {
			texts : '', // 必须，要写的文字
			width: 0, // 宽度，如果不放，则如果没遇到换行符，会一直同行写
			height: 0, // 高度 如果为0，则返回最终大小的高度
			left:0,
			top:0,
			warpW:0,
			warpH:0,
			textAlign : "left", // 文本对齐 [left|center|right]
			verticalAlign: "top", // 垂直对齐 [top|middle|bottom]
			baseLine : "middle", // 文字基线 [top|middle|bottom]
			writingMode : "lr-tb", // 书写方向：[tb-lr|bt-lr|...]t顶部，b底部，l左边，r右边
			letterSpacing : 0,  // 文字间距
			lineHeight : 1, // 文字行高
			fontSize : 100, // 文字大小
			rotate: 0, // 文字旋转

			fontFamily : 0, // 字体
			fontWeight : false, // 暂时用不上
			fontColor : "#000", // 文字颜色
			fontImage : '', // 文字图片，如果设置则会覆盖文字颜色，链接地址要带要带宽高，如xxx_210_300.png,，要不自动默认是100x100像素
			strokeSize : 0, // 描边宽度，默认是外描边
			strokeColor : "#000", // 描边颜色
			shadowX : 0, // 阴影x方向
			shadowY : 0, // 阴影y方向
			shadowSize : 0, // 阴影大小
			shadowColor : '#000000', // 阴影大小
		}

		// 默认参数
		var param = JSON.parse(JSON.stringify(defu))

		param.fontSize = data.fontSize * mini * scale;

		param.texts = (data.value || param.texts);
		param.width = (data.w * option.pageW * scale) || param.width; // 缩放抖动问题
		param.height = (data.h * option.pageH * scale) || param.height; // 缩放抖动问题
		param.left = (data.x * option.pageW * scale) || 0;
		param.top = (data.y * option.pageH * scale) || 0;
		param.warpW = (option.pageW * scale) || param.width;
		param.warpH = (option.pageH * scale) || param.height;
		param.letterSpacing = ((data.letterSpacing || param.letterSpacing) * param.fontSize); 
		param.lineHeight = ((data.lineHeight || param.lineHeight) * param.fontSize);
		param.strokeSize = ((data.strokeSize || param.strokeSize) * param.fontSize) / 10;
		param.shadowX = ((data.shadowX || param.shadowX) * param.fontSize);
		param.shadowY = ((data.shadowY || param.shadowY) * param.fontSize);
		param.shadowSize = ((data.shadowSize || param.shadowSize) * param.fontSize) / 10; 
		param.rotate = (data.rotate) || param.rotate; 

		param.textAlign = /^(left|center|right)$/.test(data.textAlign) ? data.textAlign : param.textAlign;
		param.baseLine = /^(top|middle|bottom)$/.test(data.baseLine) ? data.baseLine : param.baseLine;
		param.verticalAlign = /^(top|middle|bottom)$/.test(data.verticalAlign) ? data.verticalAlign : param.verticalAlign;
		param.writingMode = /^(lr-tb|lr-bt|rl-tb|rl-bt|tb-lr|tb-rl|bt-lr|bt-rl)$/.test(data.writingMode) ? data.writingMode : param.writingMode;
		param.fontFamily = data.fontFamily || param.fontFamily;
		param.fontWeight = false;
		param.fontColor = /(^#[0-9A-F]{6}$)|(^#[0-9A-F]{3}$)/i.test(data.color) ? data.color : param.fontColor;
		param.strokeColor = /(^#[0-9A-F]{6}$)|(^#[0-9A-F]{3}$)/i.test(data.strokeColor) ? data.strokeColor : param.strokeColor;

		// 背景、阴影
		// param.fontImage = data.fontImage || param.fontImage;
		param.shadowColor = /(^#[0-9A-F]{6}$)|(^#[0-9A-F]{3}$)/i.test(data.shadowColor) ? data.shadowColor : param.fontColor;


		// 获取文字路径
		makerGetTextsFonts(param,function(fonts){
			var img = new Image;
			var svg = getTextsSvg(param,fonts);
			img.crossOrigin = "anonymous"; //关键 解决跨域
			img.addEventListener('load', function(){callback && callback(img);},false);
			img.src = "data:image/svg+xml;charset=UTF-8," + encodeURIComponent(svg);
		});
	}

	// 画文字
	Vue.prototype.$makerDrawTexts = function(data,option,callback){

		var that = this;

		// 获取字体图片
		if(option.image === undefined){
			return this.$makerGetTextsImage(data,option,function(image){
				option.image = image;
				that.$makerDrawTexts(data,option,callback);
			});
		}

		// 数据转化，cm或百分比转化为小数点比例
		var w = (parseFloat(data.w) || 0) * option.pageW * option.warpS;
		var h = (parseFloat(data.h) || 0) * option.pageH * option.warpS;
		var x = (parseFloat(data.x) || 0) * option.pageW * option.warpS + option.warpX;
		var y = (parseFloat(data.y) || 0) * option.pageH * option.warpS + option.warpY;

		var img = option.image;


		// 图片截取比例，相对容器的宽高
		var copyW = img.width;
		var copyH = img.height;
		var copyX = 0;
		var copyY = 0;

		// 边框(图片)显示区域,相对画布
		var saveW = dataRint(option.pageW * option.warpS);
		var saveH = dataRint(option.pageH * option.warpS);
		var saveX = dataRint(option.warpX);
		var saveY = dataRint(option.warpY);

		// 截断区域，相对画布
		var clipX = Math.max(saveX, option.warpX);
		var clipY = Math.max(saveY, option.warpY);
		var clipW = Math.min(option.warpW - saveX + option.warpX, option.warpW);
		var clipH = Math.min(option.warpH - saveY + option.warpY, option.warpH);

		// 画布信息
		var canvas = option.canvas || document.createElement('canvas');
		var ctx = canvas.getContext("2d");

		canvas.width = option.canvasW;
		canvas.height = option.canvasH;


		// 画布是脏的，则清空画布
		if(canvas.dirty){
			ctx.clearRect(0,0,option.warpW,option.warpH);
		}

		// 判断文字是否在显示区域内，如果是，则显示，如果不是，则不显示
		if(saveX < canvas.width && saveX > -saveW && saveY < canvas.height && saveY > -saveH ){

			// // 旋转
			// if(data.rotate){
			// 	ctx.translate(w/2 + x, h/2 + y);
			// 	ctx.rotate(data.rotate*Math.PI/180);
			// 	ctx.translate(-w/2 - x, -h/2 - y);
			// }

			// // 剪切矩形区域
			// // 加一些偏移量，要不在旋转时会有边框出现
			// ctx.rect(Math.floor(clipX)-1, Math.floor(clipY)-1, Math.ceil(clipW)+2, Math.ceil(clipH)+2);
			// ctx.clip();

			// 半透明
			ctx.globalAlpha = data.transparent;
			ctx.drawImage(img, copyX, copyY, copyW, copyH, saveX ,saveY, saveW, saveH);
			canvas.dirty = true;
		}else{
			canvas.dirty = false;
		}

		callback && callback(canvas);
	}

	// 画图片
	Vue.prototype.$makerDrawImage = function(data,option,callback){

		var that = this;

		// 获取图片
		if(option.image === undefined){
			return this.$makerGetImageLoad(data,option,function(image){
				option.image = image;
				that.$makerDrawImage(data,option,callback);
			});
		}

		// 0.数据转化，cm或百分比转化为小数点比例
		var w = (parseFloat(data.w) || 0) * option.pageW * option.warpS;
		var h = (parseFloat(data.h) || 0) * option.pageH * option.warpS;
		var x = (parseFloat(data.x) || 0) * option.pageW * option.warpS + option.warpX;
		var y = (parseFloat(data.y) || 0) * option.pageH * option.warpS + option.warpY;
		var img = option.image;

		// 1.获得图片宽高，真实像素
		var imageW = img ? img.width : 1;
		var imageH = img ? img.height : 1;


		// 3.根据容器宽高，按图片排版方案获得图片在容器中粘贴的大小,cover:填充;contain:适应;stretch:拉伸;
		var type = ['cover','contain','stretch'].indexOf(data.render) < 0 ? 'cover' : data.render;
		var flat = type == 'cover' ? imageW / imageH > w / h : imageW / imageH < w / h; // 图片是扁的
		var baseW = type == 'stretch' ? w : flat ? h * imageW / imageH : w;
		var baseH = type == 'stretch' ? h : flat ? h : w * imageH / imageW;


		// 4.缩放偏移量
		var saveW = baseW * data.offsetW;
		var saveH = baseH * data.offsetH;
		var saveX = x + data.offsetX * w  + 0.5 * w - 0.5 * saveW;
		var saveY = y + data.offsetY * h  + 0.5 * h - 0.5 * saveH;


		// 5.截断区域
		var clipX = Math.max(option.warpX,x);
		var clipY = Math.max(option.warpY,y);
		var clipW = Math.min(option.warpX + option.warpW,x + w) - clipX;
		var clipH = Math.min(option.warpY + option.warpH,y + h) - clipY;


		// 6.获取画布信息
		var canvas = option.canvas || document.createElement('canvas');
		var ctx = canvas.getContext("2d");

		// 7.重新设置宽高，防止容器有调整了宽高，同时把坐标系初始化
		canvas.width = option.canvasW;
		canvas.height = option.canvasH;

		// 8.半透明属性；清空画布
		ctx.globalAlpha = data.transparent;
		ctx.fillStyle = '#eee';
		ctx.clearRect(0,0,option.warpW,option.warpH);
		ctx.save();

		// 9.旋转
		if(data.rotate){
			ctx.translate(w/2 + x, h/2 + y);
			ctx.rotate(data.rotate*Math.PI/180);
			ctx.translate(-w/2 - x, -h/2 - y);
		}

		// 10.图片容器裁剪
		ctx.rect(x, y, w, h);
		ctx.clip();


		// 11.如果没有图片，则显示一个灰色方框
		img ? ctx.drawImage(img,0,0,imageW,imageH,saveX,saveY,saveW,saveH) : ctx.fillRect(saveX ,saveY,saveW,saveH);

		// 剪切画布矩形区域
		ctx.restore();
		ctx.globalCompositeOperation="destination-in";
		ctx.fillRect(option.warpX, option.warpY, option.warpW, option.warpH);

		callback && callback(canvas);
	}

	// 画图片
	Vue.prototype.$makerDrawVideo = function(data,option,callback){

		var that = this;
		// 获取图片
		this.$makerGetVideoLoad(data,option,function(image){

			if(image === undefined){
				var canvas = option.canvas || document.createElement('canvas');
				callback && callback(canvas);
			}else{
				option.image = image;
				that.$makerDrawImage(data,option,callback);	
			}
		});
	}

	// 画群组
	Vue.prototype.$makerDrawGroup = function(data,option,callback){
		callback && callback(null);
	}

	// 画页面
	Vue.prototype.$makerDrawPage = function(page,option,callback){
		var that = this;
		var scale = Math.max(option.width/page.width,option.height/page.height);
		var list = (page.module || []).slice(0);
		var canvas = document.createElement('canvas');
		var ctx = canvas.getContext("2d");
		var opt = JSON.stringify({
			pageW: page.width,
			pageH: page.height,
			warpX: 0,
			warpY: 0,
			warpS: scale,
			warpW: scale * page.width,
			warpH: scale * page.height,
			canvasW: scale * page.width,
			canvasH: scale * page.height,
		});

		var drawing = function(cav){

			var next = list.shift();
			if(cav){
				ctx.drawImage(cav,0,0,cav.width,cav.height,0 ,0,canvas.width,canvas.height);
			}
			if(!next){
				// 完成
				callback && callback.call(that,canvas);
				// canvas.toBlob(function(blob) {callback && callback(blob);},"image/png", 1);

			}else if(next.type == 'texts'){
				that.$makerDrawTexts(next,JSON.parse(opt),drawing);
			}else if(next.type == 'image'){
				that.$makerDrawImage(next,JSON.parse(opt),drawing);
			}else if(next.type == 'video'){
				that.$makerDrawVideo(next,JSON.parse(opt),drawing);
			}else if(next.type == 'group'){
				that.$makerDrawGroup(next,JSON.parse(opt),drawing);
			}else{
				drawing();
			}
		}

		canvas.width = scale * page.width;
		canvas.height = scale * page.height;

		ctx.fillStyle = '#fff';
		ctx.fillRect(0, 0, canvas.width, canvas.height);
		this.$makerUpdateVersion(page);
		drawing();
	}

	Vue.prototype.$makerUpdateVersion = function(page){

		var key = [];
		var obj = {};
		var list = page.module || JSON.parse(page.data || '[]');

		key.push('w','h','x','y');
		key.push('offsetW','offsetH','offsetX','offsetY');
		key.push('imageW','imageH','rotate','transparent');
		key.push('fontSize','shadowSize','shadowX','shadowY','lineHeight','letterSpacing','strokeSize');
		key.map(function(k){obj[k] = true;})


		if(page.version != '2.0'){
			page.version = '2.0';
			list.map(function(item){

				if(item.type != 'image' && !item.render){return;}
				var title = list.filter(function(i){return i.mark == 'title' && i.pid == item.pid})[0];
				item.render = {dishes:'cover',background:'cover',image:'contain'}[item.mark] || 'cover';
				item.offsetW = 1;
				item.offsetH = 1;
				item.offsetX = 0;
				item.offsetY = 0;
				item.title = item.title || (title || {}).value || '';
				item.gid == 0 && (item.gid = '');
			});
		}

		// 数值格式化
		list.map(function(item){
			for(var i in item){
				if(item[i] === undefined){continue;}
				if(!obj[i]){continue;}
				item[i] = parseFloat(item[i])
			}
			
			// 加上锁定标签
			item.lock = !!item.lock;
		})

		page.module = list;
		delete page.data;
	}

});


