Vue.component('maker-viewer-texts', {
	template: '<canvas ref="canvas" :style="{opacity: data.transparent}"></canvas>',
	name: 'maker_texts',
	props: ['data','page','warp','select','option'],
	data: function() {
		return {
			canvas: null,
			watchcache: {option:'',data:'',drawing: false}, // 监听缓存
			pagescache: {pageW: 0,pageH: 0}, // 页面大小缓存
			fontscache: {fonts:{}}, // 文字路径缓存
			imagecache: {image:null}, // 文字图片缓存
		};
	},
	watch:{
		'page.width':function(ov,nv){this.reload();},
		'page.height':function(ov,nv){this.reload();},
		'warp.top':function(ov,nv){this.reload();},
		'warp.left':function(ov,nv){this.reload();},
		'warp.width':function(ov,nv){this.reload();},
		'warp.height':function(ov,nv){this.reload();},
		'warp.scale':function(ov,nv){this.reload();},
		'data':{deep:true,handler:function(ov,nv){this.reload();}}
	},
	methods: {

		// 获取文字路径
		getTextsFonts:function(data,option,callback){


			var that = this;
			var cache = this.fontscache;
			var value = (data.value || '').split('');
			var check = value.filter(function(i){return (cache.value || '').indexOf(i) <0});

			if(!cache.fonts || check.length >0 || cache.fontFamily != data.fontFamily){

				this.$makerGetTextsFonts(data, option, function(fonts){
					cache.value = data.value;
					cache.fontFamily = data.fontFamily;
					cache.fonts = fonts;
					that.drawCanvas(data, option, callback);
				});

				return null;
			}else {
				return cache.fonts;
			}

		},

		// 获取文字图片
		getTextsImage:function(data,option,callback){


			var that = this;
			var cache = this.imagecache;
			var pages = this.pagescache;

			var check1 = 'value,fontFamily,w,h,fontSize,lineHeight,textAlign,verticalAlign,baseLine,writingMode,letterSpacing'.split(',');
			var check2 = 'x,y,shadowSize,shadowX,shadowY,shadowColor,color,strokeSize,strokeColor'.split(',');
			var check = check1.concat(check2);
			var change = check.filter(function(i){return cache[i] != data[i]});

			pages.pageW !== option.pageW && change.push('pageW');
			pages.pageH !== option.pageH && change.push('pageH');

			if(!cache.image || change.length >0){
				this.$makerGetTextsImage(data, option,function(image){
					check.map(function(i){cache[i] = data[i]});
					cache.image = image;
					pages.pageW = option.pageW;
					pages.pageH = option.pageH;
					that.drawCanvas(data, option, callback);
				});
			}else{
				return cache.image;
			}

		},

		// 画图
		drawCanvas:function(data,option,callback){

			option.canvas = this.canvas;
			// option.fonts = this.getTextsFonts(data,option,callback) || null;
			// option.image = option.fonts ? this.getTextsImage(data,option,callback) : null;
			this.$makerDrawTexts(data, option, callback);

		},

		// 重绘
		reload:function(){

			if(!this.data){return;}
			if(!this.warp){return;}
			if(!this.page){return;}
			if(!this.page.width){return;}
			if(!this.page.height){return;}
			if(this.watchcache.drawing){return;}

			var that = this;
			var data = JSON.stringify(this.data);
			var option = JSON.stringify({
				pageW: this.page.width,
				pageH: this.page.height,
				warpW: this.warp.width * 2,
				warpS: this.warp.scale * 2,
				warpX: this.warp.left * 2,
				warpY: this.warp.top * 2,
				warpH: this.warp.height * 2,
				canvasW: this.warp.outerW * 2,
				canvasH: this.warp.outerH * 2,
			});

			if(this.watchcache.option == option && this.watchcache.data == data){return;}

			this.watchcache.data = data;
			this.watchcache.option = option;
			this.watchcache.drawing = true;

			this.drawCanvas(JSON.parse(data),JSON.parse(option),function(){
				that.watchcache.drawing = false;
				that.reload();
			});
		}
	},
	mounted: function() {
		this.canvas = this.$refs.canvas;
		this.reload();
	},

});