Vue.component('maker-viewer-video', {
	template: '<div class="maker_viewer_video" :style="canvasStyle"><div class="maker_viewer_video_warp" :style="warpStyle" ><video :src="videoUrl" :poster="videoCover" :style="itemStyle" autoplay muted loop></video></div></div>',
	props: ['data','page','warp','select','option'],
	data: function() {
		return {
			canvas: null,
			watchcache: {option:'',data:'',drawing: false}, // 监听缓存
			imagecache: {image:null}, // 文字图片缓存

			loading:false, // 是否正在更新中
			updated:false, // 是否有数据更新
			mouse:null, // 鼠标位置
			position:{}, // 位置样式
		};
	},
	watch:{
		'data':{deep:true,handler:function(ov,nv){this.setScope(this.data,this.page);}}
	},
	computed:{

		canvasStyle:function(){
			return {
				width: this.warp.width + 'px',
				height: this.warp.height + 'px',
				left: this.warp.left + 'px',
				top: this.warp.top + 'px',
			}
		},

		// 容器位置
		warpStyle:function(){

			var w = this.data.w * this.page.width * this.warp.scale;
			var h = this.data.h * this.page.height * this.warp.scale;
			var x = this.data.x * this.page.width * this.warp.scale;
			var y = this.data.y * this.page.height * this.warp.scale;
			var r = this.data.rotate || 0;

			return {
				width: w + 'px',
				height: h + 'px',
				left: x + 'px',
				top: y + 'px',
				transform: 'rotate('+r+'deg)',
			}
		},

		// 元素位置
		itemStyle:function(){

			var data = this.data;
			var w = data.w * this.page.width * this.warp.scale;
			var h = data.h * this.page.height * this.warp.scale;
			var x = data.x * this.page.width * this.warp.scale + this.warp.left;
			var y = data.y * this.page.height * this.warp.scale + this.warp.top;

			var imageW = data.imageW || 1;
			var imageH = data.imageH || 1;
			var type = ['cover','contain','stretch'].indexOf(data.render) < 0 ? 'cover' : data.render;
			var flat = type == 'cover' ? imageW / imageH > w / h : imageW / imageH < w / h; // 图片是扁的
			var baseW = type == 'stretch' ? w : flat ? h * imageW / imageH : w;
			var baseH = type == 'stretch' ? h : flat ? h : w * imageH / imageW;

			// 4.缩放偏移量
			var saveW = baseW * data.offsetW;
			var saveH = baseH * data.offsetH;
			var saveX = data.offsetX * w  + 0.5 * w - 0.5 * saveW;
			var saveY = data.offsetY * h  + 0.5 * h - 0.5 * saveH;

			return {
				width: saveW + 'px',
				height: saveH + 'px',
				left: saveX + 'px',
				top: saveY + 'px',
				opacity: data.transparent,
			}
		},
		videoUrl:function(){
			return this.$api('IMAGE') + this.data.url
		},
		videoCover:function(){
			return this.$api('IMAGE') + this.data.poster
		}

	},

	methods: {
		// 图片缩放限定范围
		setScope:function(data,page){

			var w = data.w * page.width;
			var h = data.h * page.height;
			var type = ['cover','contain','stretch'].indexOf(data.render) < 0 ? 'cover' : data.render;
			var flat = type == 'cover' ? data.imageW / data.imageH > w / h : data.imageW / data.imageH < w / h;
			var baseW = type == 'stretch' ? w : flat ? h * data.imageW / data.imageH : w;
			var baseH = type == 'stretch' ? h : flat ? h : w * data.imageH / data.imageW;

			var toW = baseW * data.offsetW;
			var toH = baseH * data.offsetH;
			var toX = data.offsetX;
			var toY = data.offsetY;

			if(toW < w){
				data.offsetX = Math.max(0.5*(toW - w)/w,Math.min(0.5*(w - toW)/w,toX));
			}else{
				data.offsetX = Math.max(0.5 - 0.5*toW/w,Math.min(0.5*toW/w - 0.5,toX));
			}

			if(toH < h){
				data.offsetY = Math.max(0.5*(toH - h)/h,Math.min(0.5*(h - toH)/h,toY));
			}else{
				data.offsetY = Math.max(0.5 - 0.5*toH/h,Math.min(0.5*toH/h - 0.5,toY));
			}
		},
		// 画图
	},
	created: function() {
	},
});