
/*! vue虚拟列表滚动插件，滚动事件响应，虚拟列表提高滚动效率 */
/*! 命名必须是多单词，比如vue-开头 */
/*! https://cn.vuejs.org/v2/style-guide/index.html#%E7%BB%84%E4%BB%B6%E5%90%8D%E4%B8%BA%E5%A4%9A%E4%B8%AA%E5%8D%95%E8%AF%8D-%E5%BF%85%E8%A6%81 */
Vue.component('vue-scroller', {
	template:'<div class="vue-scroller">'+
		'<div class="scroller-virtual">'+
			'<div class="scroller-pulldown" ref="pulldown" v-show="pulldown" :status.sync="pulldownStatus"><slot name="pulldownLoader"><i></i><span></span></slot></div>'+
			'<div class="scroller-prepend" ref="prepend"><slot name="prepend"/></div>'+
			'<ul class="scroller-list" ref="list" :style="{height:itemHeight+\'px\'}">'+
				'<li class="scroller-list-item" v-for="item in views" :key="item.index" :style="getItemStyle(item)"><slot name="content" :item="item.data" /></li>'+
			'</ul>'+
			'<div class="scroller-append" ref="append"><slot name="append"/></div>'+
			'<div class="scroller-infinite" ref="infinite" v-show="infinite" :status.sync="infiniteStatus"><slot name="infiniteLoader"><i></i><span></span></slot></div>'+
		'</div>'+
	'</div>',
	props: {
		items: {type: Array,required: true,default:[]}, // 列表
		column: {type: Number,default: 1}, // 列数
		margin: {type: Number,default: 0}, // 内容主体边距
		spacing: {type: Number,default: 0}, // 每个图片块之间距离
		pulldown: {type: Boolean,default: false},
		infinite: {type: Boolean,default: false},
		distance: {type: Number,default: 100}, // 距离底部多少时触发加载更多事件
		duration: {type: Number,default: 0}, // 下拉时长
		option: {
			type: Object, // iscroll 配置
			default: {
				scrollbars: true,
				fadeScrollbars: true,
				probeType: 3,
				bounce:true,
				click:false,
				mouseWheel: true,
				mouseWheelSpeed: 1,
                scrollX:false,
                scrollY:true,
			}
		},		
	},
	data:function() {
		return {
			iscroll: null,
			wrapperHeight:0,
			pulldownHeight:0,
			prependHeight:0,
			appendHeight:0,
			infiniteHeight:0,
			itemHeight:0,
			cache: [], // 缓存，用来判断数据有没有更新
			place: [], // 位置信息
			views: [], // 显示的界面
			count: 0, // 显示+缓存个数

			pulldownStatus:'complete',
			infiniteStatus:'nodata',
			pulldownTimer:null,
		}
	},
	watch: {
		items:function(newArr, oldArr) {
			this.initScroller();
			this.setAccumulator(); // 累加器
		}
	},

	methods: {
		
		getItemStyle:function(item){
			return {
				transform: 'translate('+ item.x + 'px,' + item.y + 'px)',
				width: item.w + 'px',
				height: item.h + 'px',
			}
		},
		fixIScrollClickInAndriod:function(){
			if (/iPhone|iPad|iPod|Macintosh/i.test(navigator.userAgent)) return false;
			if (/Chrome/i.test(navigator.userAgent)) return (/Android/i.test(navigator.userAgent));
			if (/Silk/i.test(navigator.userAgent)) return false;
			if (/Android/i.test(navigator.userAgent)) {
				var s = navigator.userAgent.substr(navigator.userAgent.indexOf('Android') + 8, 5);
				//return parseFloat(s[0]+s[3]) < 44 ? false : true}
				return parseFloat(s[0] + s[2] + s[4]) <= 442 && parseFloat(s[0] + s[2] + s[4]) > 430 ? true : false
			}
		},

		// 下拉刷新
		triggerPulldown:function() {

			var that = this;
			var is_begin = this.pulldownStatus == 'begin';
			var is_trigger = this.pulldownStatus == 'trigger';
			var is_refresh = this.pulldownStatus == 'refresh';
			var is_complete = this.pulldownStatus == 'complete';
			var is_error = this.pulldownStatus == 'error';

			if (!this.pulldown){return};
			if (is_refresh){return};
			if (is_error){return};

			// 拖动距离小于刷新容器距离，则展示begin
			// 超过距离由触发trigger，提示“停留2秒刷新”
			// 2秒后刷新refresh
			// 刷新完成complete
			// 刷新完成后拖动不再触发，一直到下一次拖动

			if(this.iscroll.y <= this.pulldownHeight){

				// begin、trigger、complete

				// 触发中
				if(is_trigger){

					this.pulldownStatus = 'refresh';
					this.$emit('onpullrefresh');
					
					// this.pulldownTimer && clearTimeout(this.pulldownTimer);
					// this.pulldownStatus = 'begin';
				}else if(is_complete){
					// 间隔时间查询iscroll，如果y<0,则还原
					this.pulldownTimer && clearTimeout(this.pulldownTimer);
					this.pulldownTimer = setTimeout(function(){
						that.pulldownTimer = null;
						that.iscroll.y <= 0 && (that.pulldownStatus = 'begin');
					},100);

				}else{
					// 正在开始
				}

			}else if(!is_trigger && !is_complete){

				this.pulldownStatus = 'trigger';

				// this.pulldownTimer && clearTimeout(this.pulldownTimer);
				// this.pulldownTimer = setTimeout(function(){
				// 	that.pulldownTimer = null;
				// 	if(that.pulldownStatus == 'trigger'){
				// 		that.pulldownStatus = 'refresh';
				// 		that.$emit('onpullrefresh');
				// 	}
				// },this.duration || 0);
			}
		},

		// 加载更多
		triggerInfinite:function() {
			if (!this.infinite) {return;}
			if(this.infiniteStatus == 'complete'){return;}
			if(this.infiniteStatus == 'nodata'){return;}
			if(this.infiniteStatus == 'loading'){return;}
			this.infiniteStatus = 'loading';
			this.$emit('onloadmore');
		},

		// 判断是否在容器范围内
		insideView: function(item){
			var y = -this.iscroll.y;
			var h = this.wrapperHeight;
			var pre = this.prependHeight;

			return item.b > y - pre && item.y < y + h - pre;
		},

		// 选中视图
		selectView:function(){


			var that = this;
			var toup = this.iscroll.directionY == -1; // 向下滑
			var views = this.views.slice(0);
			var place = this.place;
			var count = this.count;
			var toend =  place.length - 1;
			var oldview = {}; // 原来的数据
			var newview = {};
			var insert = []; // 要插入的位置

			if(place.length == 0){
				views.length = 0;
			}else if(toup){
				// 优先展示前面的数据
				var index = views.length >0 ? views.slice(-1)[0].index : place.slice(-1)[0].index || toend;
				index = Math.min(index + count, toend); // 加上缓冲
				while (place[index] && !this.insideView(place[index])) {index -= 1;}
				if(index + 1 - count < 0){
					views = place.slice(0,count);
				}else{
					views = place.slice(index + 1 - count,index + 1);
				}
			}else{
				// 优先展示后面的数据
				var index = views.length >0 ? views[0].index : place[0].index || 0;
				index = Math.max(0,index - count); // 加上缓冲
				while (place[index] && !this.insideView(place[index])) {index += 1;}

				if(index + count >= place.length){
					views = place.slice(Math.max(place.length - count,0));
				}else{
					views = place.slice(index,index + count);
				}				
			}

			// 只改变变化的部分
			// 获取新增的部分
			// 判断哪些未存在的，则更新
			views.map(function(i){newview[i.index] = i});
			this.views.map(function(i,idx){
				oldview[i.index] = i;
				!newview[i.index] && insert.push(idx);
			});

			// 添加或变化
			views.map(function(item){

				if(oldview[item.index]){
					return;
				}else{
					if(insert.length >0){
						that.$set(that.views,insert.shift(),item);
					}else{
						that.views.push(item);
					}
				}
			});


			// 如果有多出来的要删除掉
			this.views.map(function(i,idx){!newview[i.index] && that.views.splice(that.views.indexOf(i),1);});

		},
		
		// 滚动中
		handleScrollMove:function() {

			if(!this.iscroll){return;}

            this.$emit('onscroll', this.iscroll);
			if(this.place.length == 0){return;}

			if (this.iscroll.directionY > 0 && this.iscroll.maxScrollY > this.iscroll.y - this.distance) {
				this.triggerInfinite();
			} else if (this.pulldown && this.iscroll.y > 0) {
				this.triggerPulldown();
			} else {

				this.selectView();
				
			}
		},
		// 滚动完成
		handleScrollEnd:function() {
			this.$emit('onscrollend', this.iscroll);
		},
		// 滚动开始
        handleScrollStart:function() {
			this.$emit('onscrollstart', this.iscroll);
		},

		// 初始化滚动条
		initScroller:function(){

			if(this.iscroll){
				return;
			}else if(this.$el){
				this.wrapperHeight = this.$el.offsetHeight;
				this.prependHeight = this.$refs.prepend.offsetHeight;
				this.appendHeight = this.$refs.append.offsetHeight;
				this.pulldownHeight = this.$refs.pulldown.offsetHeight;
				this.infiniteHeight = this.$refs.infinite.offsetHeight;
				this.option.click = this.fixIScrollClickInAndriod(); // 响应点击事件

				this.iscroll = new IScroll(this.$el, this.option)
				this.iscroll.on('scroll', this.handleScrollMove)
				this.iscroll.on('scrollEnd', this.handleScrollEnd)
				this.iscroll.on('scrollStart', this.handleScrollStart)
			}
		},

		// 计算位置
		setAccumulator:function(){

			// 判断哪些添加了，哪些删除了，哪些位置改了
			var that = this;

			var cache = this.cache;
			var items = this.items;
			var place = this.place;
			var spacing = this.spacing || 0; // 每个图片块之间距离
			var margin = this.margin || 0;// 内容主体边距
			var warph = this.wrapperHeight; // 容器高度
			var count = Math.max(cache.length,items.length);
			var buffer = 3; // 几倍的显示区
			var index = -1;

			// 找不一样的位置，从不一样的位置开始计算
			for(var i = 0; i<count; i++){
				if(!cache[i] || !items[i] || cache[i] != items[i] || cache[i].height != items[i].height){
					index = Math.max(index - 1,0);
					break;
				}
			}

			if(index >= 0){

				var count = parseInt(this.column) || 1; // 列数
				var width = parseInt((this.$el.offsetWidth - margin * 2 - (count - 1) * spacing)/count);
				var height = 0; 

				// 把其他的删除掉
				items.slice(index).map(function(item,idx){

					var series = new Array(count).join('0').split('0'); // 各列最低的位置
					var column = {col:0,top:null}; // 当前列的位置
					var current = place[index + idx] || {}

					for(var last = index + idx - 1; last>=0; last--){

						var the = place[last]; // 选中的项目
						var col = the.col; // 所在的列
						var top = parseFloat(the.y + the.h) || 0;

						series[col] === '' && (series[col] = top); // 如果有该列没有数据，则放置上去
						if(series.indexOf('') < 0){break;}
					}

					// 找最低的，或为 null 的列
					series.map(function(top,idx){
						top = parseInt(top) || 0;
						if(column.top === null || top < column.top){
							column.top = top;
							column.col = idx;	
						}
					});

					current.data = item;
					current.index = index + idx;
					current.col = column.col;
					current.h = parseFloat(item.height) || 0;
					current.w = width;
					current.x  = margin + column.col * (current.w + spacing);
					current.y = Math.max(0,(column.top || 0) + spacing);
					current.b = current.y + current.h; // 距离底部

					// margin: {type: Number,default: 10}, // 内容主体边距
					// spacing: {type: Number,default: 10}, // 每个图片块之间距离

					place[current.index] = current;
					height = Math.max(current.y + current.h,height);

				});

				this.pulldownStatus = this.iscroll.y <= 0 ? 'begin' : 'complete';
				this.infiniteStatus = items.length == 0 ? 'nodata' : 'begin';
				this.cache = items.slice(0);
				this.count = Math.min(place.filter(function(i){return i.y <= warph}).length * buffer,items.length);
				this.$set(this.place,'length',items.length);
                
				if(this.itemHeight != height){

					// 高度发生变化
					var y = this.iscroll.y;
					this.itemHeight = height;
					this.$refs.list.style.height = height + 'px';
					this.iscroll.refresh();
					this.views.splice(0,this.views.length);
					this.selectView();

					if(y>0){
						this.pulldownStatus = 'complete';
						this.iscroll.scrollTo(0,y);
					}


				}else{
					// 高度未发生变化，但是可能元素变了
					this.selectView();
				}
			}
		},

		// 结束加载更多
		infiniteComplete:function(){
			this.infiniteStatus = this.items.length == 0 ? 'nodata' : 'complete';
		},
		refresh:function(){
			this.iscroll.refresh();
			this.iscroll.scrollTo(0,0);
			this.views.length = 0;
			this.selectView();
		},
		
	},
	mounted:function() {
		this.cache = [];
		this.initScroller();
		this.setAccumulator();
	},
	beforeDestroy:function(){this.iscroll = this.iscroll && this.iscroll.destroy() && null;},
});