
/*! vue虚拟列表滚动插件，滚动事件响应，虚拟列表提高滚动效率 */
/*! 命名必须是多单词，比如vue-开头 */
/*! https://cn.vuejs.org/v2/style-guide/index.html#%E7%BB%84%E4%BB%B6%E5%90%8D%E4%B8%BA%E5%A4%9A%E4%B8%AA%E5%8D%95%E8%AF%8D-%E5%BF%85%E8%A6%81 */
Vue.component('vue-scroller', {
	template:
	'<div class="vue-scroller">'+
		'<div class="scroller-virtual">'+
			'<div class="scroller-refresh" v-if="pulldown" v-show="pullState.length > 0" :style="getPullerStyle">'+
				'<slot name="pullRefresh">'+
					'<i class="iconfont" :class="getPullerClass"></i>'+
					'<span class="text-default">{{pullText}}</span> '+
				'</slot>'+
			'</div>'+
			'<div class="scroller-prepend"><slot name="prepend"/></div>'+
			'<ul class="scroller-list">'+
				'<li class="scroller-list-item" v-for="item in pool" :key="item[uniqueKey]" :style="getItemStyle(item)">'+
					'<slot name="content" :item="item" />'+
				'</li>'+
			'</ul>'+
			'<div class="scroller-append" :style="getSpinnerStyle"><slot name="append"/></div>'+
			'<div class="scroller-infinite" v-if="infinite" v-show="infiniteLoading" :style="getSpinnerStyle">'+
				'<slot name="infiniteLoader">'+
					'<i class="spinner-default"></i>'+
					'<span class="text-default">加载中...</span>'+
				'</slot>'+
			'</div>'+
		'</div>'+
	'</div>',
	props: {
		items: {type: Array,required: true},
		uniqueKey: {type: String,required: true,default: 'id'},
		infinite: Boolean,
		pulldown: Boolean,
		variable: {type: Boolean,default: false},
		distance: {type: Number,default: 50},
		bufferSize: [String, Number],
		loadMore: Function,
		pullDuration: {type: Number,default: 2000}, // 下拉时长
		pullRefresh: Function,
		pulldownText: {
			type: Object,
			default: {
				begin: '下拉刷新',
				trigger: '2s后释放更新',
				refresh: '加载中...',
				complete: '加载完成',
				error: '加载失败'
			}
		},
		iscrollOptions: {
			type: Object,
			default: {
				scrollbars: true,
				interactiveScrollbars: true,
				probeType: 3,
				bounce:true,
				click:false,
				mouseWheel: true,
				mouseWheelSpeed: 1,
			}
		},
	},
	data:function() {
		return {
			myScroll: null,
			viewInited: false,
			wrapperHeight: 0,
			prependHeight:0, // 前缀高度
			appendHeight:0, // 后缀高度
			itemHeight: 0,
			buffer: 0,
			poolLength: 0,
			pool: [],
			infiniteLoading: false,
			infiniteComplete: false,
			pullTimer: null,
			pullerTop: 0,
			pullState: '',
			accumulator: 0,
			oriItemLength: 0,
			scrolledItem: null
		}
	},
	created:function() {
		this.buffer = this.bufferSize || 5
		this.pool = this.items.slice(0, this.buffer)
	},
	mounted:function() {
		this.$nextTick(function(){
			this.initScroller()
		})
	},
	beforeDestroy:function(){
		this.destroy();
	},
	watch: {
		items:function(newArr, oldArr) {

			this.pool = this.items.slice(0, this.buffer)
			this.$nextTick(function() {

				this.generateItemAccumulator(true);
				// 如果新的高度比旧的
				!this.viewInited && this.initScrollView();
			})
			if (this.pool.length < this.poolLength) {
				var index = this.items.indexOf(this.pool[0])
				this.pool = this.items.slice(index, this.poolLength)
			}
		}
	},
	computed: {
		getPullerStyle:function() {
			var top = -this.pullerTop
			return {
				transform: 'translate(0, ' + top + 'px)'
			}
		},
		getPullerClass:function() {
			return {
				'icon-pulldown': this.pullState === 'begin' || this.pullState === 'trigger',
				'icon-pullup': this.pullState === 'trigger',
				'spinner-default': this.pullState === 'refresh',
				'icon-complete': this.pullState === 'complete',
				'icon-error': this.pullState === 'error'
			}
		},
		getSpinnerStyle:function() {
			var top = this.variable ? this.accumulator : this.itemHeight * this.items.length
			return {
				transform: 'translate(0, ' + top + 'px)'
			}
		},
		pullText:function() {
			return this.pulldownText[this.pullState] || ''
		}
	},
	methods: {
		generateItemAccumulator:function(init) {
			var that = this;
			if (!this.variable) return
			if (init) {
				this.accumulator = 0
				this.items.forEach(function(item) {
					that.$set(item, '_top', that.accumulator + that.prependHeight)
					that.accumulator += item.height;
				})
			} else {
				this.items.slice(this.oriItemLength).forEach( function(item) {
					that.$set(item, '_top', that.accumulator + that.prependHeight)
					that.accumulator += item.height
				})
			}
		},
		fixIScrollClickInAndriod:function(){
			if (/iPhone|iPad|iPod|Macintosh/i.test(navigator.userAgent)) return false;
			if (/Chrome/i.test(navigator.userAgent)) return (/Android/i.test(navigator.userAgent));
			if (/Silk/i.test(navigator.userAgent)) return false;
			if (/Android/i.test(navigator.userAgent)) {
				var s = navigator.userAgent.substr(navigator.userAgent.indexOf('Android') + 8, 5);
				//return parseFloat(s[0]+s[3]) < 44 ? false : true}
				return parseFloat(s[0] + s[2] + s[4]) <= 442 && parseFloat(s[0] + s[2] + s[4]) > 430 ? true : false
			}
		},
		initScroller:function() {
			
			this.prependHeight = this.$el.querySelector('.scroller-prepend').clientHeight;
			this.appendHeight = this.$el.querySelector('.scroller-append').clientHeight;


			this.iscrollOptions.click = this.fixIScrollClickInAndriod();

			this.myScroll = new IScroll(this.$el, this.iscrollOptions)
			this.myScroll.on('scroll', this.handleScrollEvent)
			this.myScroll.on('scrollEnd', this.handleScrollEndEvent)
			this.myScroll.on('pullDownEnd', this.handlePullDownEndEvent) // 并没有这个事件

			this.generateItemAccumulator(true);
			this.initScrollView()
			
		},
		initScrollView:function() {
			this.wrapperHeight = this.$el.clientHeight;

			if (this.items.length === 0) return
			if (!this.variable) {
				this.itemHeight = this.$el.querySelector('.scroller-list-item').offsetHeight;
				this.poolLength = Math.ceil((this.wrapperHeight-this.prependHeight - this.appendHeight) / this.itemHeight) + 2 * this.buffer;
				this.pool = this.items.slice(0, this.poolLength)
				
				this.resetScroller();
				this.updateScrollView()
			} else {
				var initSize = this.getScrolledIndex(this.wrapperHeight)
				this.poolLength = initSize + 2 * this.buffer
				this.pool = this.items.slice(0, this.poolLength)
				this.resetScroller()
			}
			this.viewInited = true
		},
		
		destroy:function() {
			if (this.myScroll) {
				this.myScroll.destroy()
				this.myScroll = null
			}
		},
		refresh:function() {
			if (this.myScroll) {
				this.destroy()
				this.$nextTick(function() {
					this.generateItemAccumulator(true);
					this.resetParams();
					this.initScroller();
					this.resetScroller();
				})
			}
		},
		resetParams:function() {
			this.infiniteLoading = false
			this.infiniteComplete = false
		},
		getScrolledIndex:function(target) {
			var start = 0
			var end = this.items.length - 1
			while (start <= end) {
				var mid = parseInt(start + (end - start) / 2)
				var item = this.items[mid]
				if (target >= item._top && target < item._top + item.height) {
					return mid
				} else if (target < item._top) {
					end = mid - 1
				} else {
					start = mid + 1
				}
			}
			return -1
		},
		handleScrollEvent:function() {

			if(!this.myScroll){return;}

			if (!this.infiniteComplete && !this.infiniteLoading && this.myScroll.directionY > 0 && this.myScroll.maxScrollY > this.myScroll.y - this.distance) {
				this.triggerLoadmore();
			} else if (this.pulldown && this.myScroll.y > 0) {
				this.triggerPulldownRefresh()
			} else {
				this.updateScrollView();
				this.$emit('onscroll', this.myScroll);
			}
		},
		handleScrollEndEvent:function() {
			this.$emit('onscrollend', this.myScroll);
			if (this.pullState === 'complete' || this.pullState === 'error' || this.pullState === 'begin') {
				this.pullState = ''
				this.myScroll && (this.myScroll.pullState = '');
			}
		},
		handlePullDownEndEvent:function() {
			if (this.pullState === 'trigger') {
				this.pullState = 'refresh'
				this.myScroll.pullState = 'refresh'
				this.$emit('onpullrefresh', this.pullStateManager)
			}
		},
		triggerLoadmore:function() {
			if (!this.infinite) return
			this.isPulling = false
			this.infiniteLoading = true
			this.oriItemLength = this.items.length
			this.$nextTick(function() {
				this.resetScroller(this.$el.querySelector('.scroller-infinite').offsetHeight)
			})
			this.$emit('onloadmore', this.infiniteStateManager)
		},
		triggerPulldownRefresh:function() {
			if (!this.pulldown) return;
			if (this.infiniteLoading || this.pullState === 'refresh' || this.pullState === 'complete' || this.pullState === 'error') return

			if (!this.pullState || this.myScroll.y <= this.pullerTop) {
				this.pullState = 'begin'
				this.myScroll.pullerHeight = 0
				if (!this.pullerTop) {
					this.$nextTick(function() {
						this.pullerTop = this.$el.querySelector('.scroller-refresh').offsetHeight
					})
				}
			} else if (this.pullState != 'trigger' && this.myScroll.y > this.pullerTop) {

				var that = this;
				this.pullState = 'trigger'
				this.myScroll.pullerHeight = this.pullerTop;

				this.pullTimer && clearTimeout(this.pullTimer);
				this.pullTimer = setTimeout(function(){
					that.pullTimer = null;
					if(that.pullState == 'trigger'){
						that.pullState = 'refresh';
						that.$emit('onpullrefresh', that.pullStateManager)
					}
				},this.pullDuration || 0);
			}
		},
		infiniteStateManager:function(state) {
			switch (state) {
				case 'loaded':
					this.infiniteLoading = false
					this.generateItemAccumulator(false)
					this.$nextTick(function() {
						this.resetScroller()
						this.updateScrollView()
					})
					break
				case 'error':
					this.infiniteLoading = false
					this.$nextTick(function() {
						this.resetScroller()
					})
					break
				case 'complete':
					this.infiniteLoading = false
					this.infiniteComplete = true
					this.generateItemAccumulator(true)
					this.$nextTick(function() {
						this.resetScroller()
					})
					break
			}
		},
		pullStateManager:function(state) {
			if (state === 'complete') {
				this.myScroll.pullerHeight = 0
				this.pullState = 'complete'
				this.generateItemAccumulator(true)
				setTimeout(function() {
					this.pullState = '';
					this.resetScroller(null, 600)
					this.resetParams()
				}, 500)
			} else {
				this.myScroll.pullerHeight = 0
				this.pullState = 'error'
				setTimeout(function() {
					this.pullState = '';
					this.resetScroller(null, 600)
					this.resetParams()
				}, 500)
			}
		},
		resetScroller:function(loaderHeight, time) {
			var h = loaderHeight || 0;
			if (!this.variable) {
				this.myScroll.scrollerHeight = this.itemHeight * this.items.length + h + this.prependHeight + this.appendHeight;
				this.myScroll.maxScrollY = -this.itemHeight * this.items.length + this.wrapperHeight - h
			} else {
				this.myScroll.scrollerHeight = this.accumulator + h + this.prependHeight + this.appendHeight;
				this.myScroll.maxScrollY = this.wrapperHeight - this.accumulator - h
			}


			this.$el.children[0].style.height = this.myScroll.scrollerHeight + 'px';
			this.myScroll.refresh(time)

		},
		updateScrollView:function() {
			if (!this.variable) {
				var scrolledLength = Math.max(Math.floor(-this.myScroll.y / this.itemHeight) - this.buffer, 0)
				var majorPhase = Math.floor(scrolledLength / this.pool.length)
				var majorLength = scrolledLength % this.pool.length
				var i = 0
				var top = 0;
				while (i < this.pool.length) {
					top = majorPhase * this.pool.length * this.itemHeight + i * this.itemHeight;
					if (i < majorLength) {
						top += this.itemHeight * this.pool.length
					}
					if (i < this.pool.length && this.pool[i]._top !== top) {
						this.updateItem(i, top);
					}
					i++
				}
			} else {
				var scrolledIndex = this.getScrolledIndex(-this.myScroll.y)
				var scrolledLength = Math.max(scrolledIndex - this.buffer, 0)
				var majorPhase = Math.floor(scrolledLength / this.pool.length)
				var majorLength = scrolledLength % this.pool.length
				var i = 0
				var newIndex = 0
				while (i < this.pool.length) {
					newIndex = majorPhase * this.pool.length + i
					if (i < majorLength) {
						newIndex += this.pool.length
					}
					if (newIndex < this.items.length && this.pool[i] !== this.items[newIndex]) {
						this.updateItem(i, newIndex)
					}
					i++
				}
			}
		},
		updateItem:function(i, top) {
			if (!this.variable) {
				var index = top / this.itemHeight
				if (index < this.items.length) {
					var item = this.items[index]
					item._top = top + this.prependHeight;
					this.$set(this.pool, i, item)
				}
			} else {
				var item = this.items[top]
				this.$set(this.pool, i, item)
			}
		},
		getItemStyle:function(item) {
			return {
				transform: 'translate(0, ' + item._top + 'px)'
			}
		}
	}
});