Vue.component('yunscreen-setting', {
    template:
        '<div class="title_button">' +
        "<a @click=\"actionSheet.show=true\" v-text=\"actionSheet.show && actionSheet.type ==  'settting' ? '取消' : '设置'\"></a>" +
        '<van-action-sheet v-model="actionSheet.show" :actions="actionSheet.actions" cancel-text="取消" close-on-click-action @select="onActionSheetSelect" @cancel="actionSheet.show=false"></van-action-sheet>' +
        '</div>',
    props: {
        deviceId: { type: Number, default: 0 },
        detail: { type: Object, default: null },
    },
    data: function () {
        return {
            actionSheet: {
                type: 'setting',
                state: null,
                show: false,
                actions: [
                    { type: 'reName', name: '修改名称' },
                    { type: 'rotation', name: '旋转' },
                    { type: 'reload', name: '重启' },
                    { type: 'shutdown', name: '关机' },
                    { type: 'timingswitch', name: '定时开关机' },
                    { type: 'upgrade', name: '升级系统' },
                    { type: 'clear', name: '清空数据' },
                    { type: 'unBind', name: '解绑云屏' },
                ],
            },
        }
    },
    methods: {
        // 解绑
        unBind: function () {
            var that = this
            var deviceNo = this.detail.deviceNo
            if (!deviceNo) {
                this.$toast('deviceNo不存在')
                return
            }
            this.$confirm('确定解绑吗？', ['取消', '确定解绑'], function (value) {
                if (value != '确定解绑') {
                    return
                }
                that.loading = true
                that.$ajax('yunscreen_unbind', { deviceNo }, function (json) {
                    that.loading = false
                    if (json.code == 200) {
                        that.$emit('global:yunrecipe_list_update', true)
                        that.$toBack()
                    } else {
                        that.$toast(json.message)
                    }
                })
            })
        },

        // 重命名
        reName: function (item) {
            var that = this
            var label = '云屏名称'
            var value = this.detail.name || ''
            var html =
                '<label><input placeholder="' +
                label +
                '" type="text" name="title" autofocus="autofocus" value="' +
                value +
                '" /></label>'

            this.$confirm(html, ['确定', '取消'], function (text, warp) {
                if (text == '确定') {
                    var form = warp.querySelector('form')
                    var input = warp.querySelector('input')
                    var title = (form.title.value || '').replace(/\s/gi, '')

                    if (!title) {
                        this.$toast('名称不能为空')
                        setTimeout(function () {
                            input && input.focus && input.focus()
                        }, 600)
                        return true
                    } else {
                        that.loading = true
                        that.$ajax(
                            'yunscreen_rename',
                            { deviceId: that.$urlParam('deviceId'), name: title },
                            function (json) {
                                that.loading = false
                                that.showEdit = false
                                if (json.code == 200) {
                                    that.getDetail()
                                    that.$emit('global:yunrecipe_list_update', true)
                                } else {
                                    that.$toast(json.message)
                                }
                            }
                        )
                    }
                }
            })
        },

        // 旋转
        rotation: function () {
            var that = this
            var rotation = parseInt(this.detail.rotationAngle) || 0
            var direction = [0, 90]
            var label = direction.map(function (item) {
                var value = item
                // var temp = '<label><input type="radio" name="rotation" {c} value="{v}"><em>{v}°</em></label>';
                var temp =
                    '<label><input type="radio" name="rotation" {c} value="{v}"><em>{vText}</em></label>'

                // 0 和 90 这里用文字显示成 横屏 和 竖屏
                var valuemap = { 0: '横屏', 90: '竖屏' }
                temp = temp.replace(/\{v\}/gi, value)
                temp = temp.replace(/\{vText\}/gi, valuemap[value] || value)
                temp = temp.replace(/\{c\}/g, item == rotation ? 'checked' : '')
                return temp
            })

            var html =
                '<div class="rotation-radio"><p>此操作将会<span class="red">解绑模板</span>，旋转完成后需要您<span class="red">重新添加</span>。您确认旋转吗？</p>' +
                label.join('') +
                '</div>'

            this.$confirm(html, ['确定', '取消'], function (text, warp) {
                if (text == '确定') {
                    var form = warp.querySelector('form')
                    var value = parseInt(form.rotation.value) || 0

                    if (value == rotation) {
                        // 和原来的一样，不用处理
                        // return true;
                    } else {
                        var change = direction.indexOf(rotation) % 2 != direction.indexOf(value) % 2
                        var width = change ? this.detail.height : this.detail.width
                        var height = change ? this.detail.width : this.detail.height

                        var req = {
                            id: parseInt(that.$urlParam('deviceId')), // number  必须,设备id
                            width: parseInt(width), //  number  必须,宽
                            height: parseInt(height), // number  必须,高
                            rotationAngle: value, //  number  必须,旋转角度
                        }

                        that.loading = true
                        that.$ajax('yunscreen_rotate', req, function (json) {
                            that.loading = false
                            if (json.code == 200) {
                                that.getDetail()
                                that.$emit('global:yunrecipe_list_update', true)
                            } else {
                                that.$toast(json.message)
                            }
                        })
                    }
                }
            })
        },

        // 定时开关时间
        pickerTimingHandle: function (type, confirm) {
            var pickerTiming = this.pickerTiming
            var that = this

            if (type == 'timeStart') {
                if (confirm) {
                    pickerTiming.timeEndShow = true
                } else {
                    pickerTiming.timeStartShow = false
                }
            } else if (type == 'timeEnd') {
                if (confirm) {
                    pickerTiming.timeStartShow = false
                    pickerTiming.timeEndShow = false

                    var request = {
                        type: 'timingBoot',
                        id: that.$urlParam('deviceId'),
                        onAt: pickerTiming.timeStart,
                        stopAt: pickerTiming.timeEnd,
                    }
                    console.log(request)
                    that.loading = true
                    that.$ajax('yunscreen_sendOrder', request, function (json) {
                        that.loading = false
                        if (json.code == 200) {
                            that.$toast(json.message || '设置成功')
                        } else {
                            that.$toast(json.message)
                        }
                    })
                } else {
                    pickerTiming.timeEndShow = false
                }
            }
        },

        onActionSheetSelect: function (item) {
            var that = this
            if (item.type == 'unBind') {
                // 解绑
                this.unBind()
            } else if (item.type == 'reName') {
                // 重命名
                this.reName()
            } else if (item.type == 'rotation') {
                // 旋转
                this.rotation()
            } else if (item.type == 'timingswitch') {
                // 定时开关
                this.$toLink('yunscreen_switch.html', {
                    deviceId: this.$urlParam('deviceId'),
                })
                // this.pickerTiming.timeStartShow = true
            } else {
                // item.type == 'display'
                // item.type == 'preview'
                // item.type == 'reload'
                // item.type == 'upgrade'
                // item.type == 'shutdown'

                var that = this
                var request = {
                    type: item.type,
                    id: this.$urlParam('deviceId'),
                }

                that.$ajax('yunscreen_sendOrder', request, function (json) {
                    if (json.code == 200) {
                        that.$toast(json.message || '发送成功')
                    } else {
                        that.$toast(json.message)
                    }
                })
            }
        },
    },
    created: function () {},
})
