Vue.component('yunscreen-used', {
    template:
        '<div class="program" v-loading="loading">' +
        '<vue-scroller ref="scroller" :items="pages" :infinite="true" :pulldown="true" :column="2" :margin="15" :spacing="10" @onloadmore="pagesUpdata(false)" @onpullrefresh="pagesUpdata(true)" >' +
        '<template slot="prepend"><div style="height: 5px"></div></template>' +
        '<template slot="append"><div style="height: 5px"></div></template>' +
        '<template slot="content" slot-scope="props">' +
        '<div class="program-item" :class="{selected:selectable && props.item.selected,selectable: selectable,paid: props.item.payStatus != 0}">' +
        '<figure :style="{paddingBottom:props.item.paddingBottom}" v-loading="props.item.building ? \'正在生成\': \'\'" @click="toPlay(props.item)" >' +
        '<img v-if="props.item.thumb" :src="$cache(props.item.thumb,\'IMAGE\')"/>' +
        '<span><i class="iconfont icon-play"></i></span>' +
        '<h6 v-show="selectable"><i v-text="props.item.selectindex"></i></h6>' +
        '</figure>' +
        '<h3 v-text="props.item.name || \'未命名\'"></h3>' +
        '<template v-if="props.item.amountInfo">' +
        '<h4 v-if="props.item.payStatus == 0 "><span>待支付</span><strong v-text="parseInt(props.item.amountInfo.amount)" ></strong><small v-text="(props.item.amountInfo.amount%1).toFixed(2).replace(/^0/,\'\')" ></small><span>元</span><button @click="toEdit(props.item)"><i class="iconfont icon-bianji"></i><span>编辑</span></button></h4>' +
        // '<h4 v-if="props.item.payStatus == 0 "><span>待支付</span><strong v-text="parseInt(props.item.amountInfo.amount)" ></strong><small v-text="(props.item.amountInfo.amount%1).toFixed(2).replace(/^0/,\'\')" ></small><span>元</span><a type="pay" @click="toOrder(props.item.id)" ><i class="iconfont icon-fukuantongzhi"></i><span>去支付</span></a></h4>' +
        // '<h4 v-else><span>已支付</span><button @click="toPlay(props.item)"><i class="iconfont icon-bianji"></i><span>去播放</span></button></h4>' +
        '<h4 v-else><span v-text="props.item.deviceList.length+\'个云屏在使用\'"></span><button @click="toEdit(props.item)"><i class="iconfont icon-bianji"></i><span>编辑</span></button></h4>' +
        '</template>' +
        '</div>' +
        '</template>' +
        '</vue-scroller>' +
        '<div class="scroller-nodata" style="display: none" v-show="!loading && pages.length == 0" >' +
        '<p>' +
        '<i class="iconfont icon-wushuju"></i>' +
        '<strong>暂无云菜谱</strong>' +
        '<span>(在『一键制作中』选择云菜谱模板开始制作吧)</span>' +
        '</p>' +
        '</div>' +
        '<div class="main_collect_select" v-show="selectable" style="display: none" >' +
        '<span @click="selectAll()" :class="{active:selectall}"><i class="iconfont icon-checkboxselect"></i><em></em></span>' +
        '<strong style="text-align: left">选中{{selected.length}}个云菜谱</strong>' +
        '<b @click="toRemove()">删除</b>' +
        '<a @click="toOrder()">批量支付</a>' +
        '</div>' +
        '</div>',

    props: {
        selectable: { type: Boolean, default: false },
    },
    data: function () {
        return {
            loading: true,
            form: { currentPage: 0, showCount: 50 },
            pages: [], // 每天数据列表
            nomore: false,
            selected: [],
            selectall: false,
        }
    },
    methods: {
        // 获取分页
        pagesUpdata: function (refresh, callback) {
            var that = this

            if (this.nomore && !refresh) {
                return
            }
            if (refresh) {
                this.loading = true
            }

            // 分页调整
            this.form.currentPage = refresh ? 1 : this.form.currentPage + 1
            // ajax
            this.$ajax('yunscreen_program_list', this.form, function (response) {
                var data = (response || {}).data || {}
                var list = data.records || []
                var pages = that.pages // 按日期分
                var width = (window.screen.width - 40) / 2 // 两个并一行

                // 如果刷新，则清空原数据
                if (refresh) {
                    let building = pages.filter(function (item) {
                        return item.building
                    })
                    pages.length = 0
                    pages.push.apply(pages, building)
                }

                list.map(function (item, index) {
                    item.thumbW = parseInt(item.width)
                    item.thumbH = parseInt(item.height)

                    item.paddingBottom = (100 * item.thumbH) / item.thumbW + '%'
                    item.height = (width * item.thumbH) / item.thumbW + 60
                    item.selected = false
                })

                // 添加
                pages.push.apply(pages, list)

                that.loading = false
                that.nomore = list.length < that.form.showCount
                that.nomore &&
                    that.$nextTick(function () {
                        that.$refs.scroller.infiniteComplete()
                    })
                callback && callback.call(that)
            })
        },

        // 判断选中的
        selectFilter: function () {
            var that = this
            this.selected = this.pages.filter(function (i) {
                return i.selected
            })
            this.pages.map(function (item) {
                item.selectindex = item.selected ? that.selected.indexOf(item) + 1 : ''
            })

            this.selectall = this.selected.length >= this.pages.length
        },

        // 全选全不选
        selectAll: function () {
            var selected = !this.selectall
            this.pages.map(function (item) {
                item.selected = selected
            })
            this.selectFilter()
        },

        // 删除
        toRemove: function () {
            var that = this
            var ids = this.selected
                .map(function (i) {
                    return i.id
                })
                .join(',')
            var playing = this.selected.filter(function (i) {
                return i.deviceList.length > 0
            })

            // 如果有视频正在播放中，则提示不能删除
            if (playing.length > 0) {
                this.$confirm('云屏正在使用的云菜谱不能删除，请下架云菜谱后再删除', ['确定'])
            } else if (!ids) {
                this.$toast('请选择云菜谱')
            } else {
                this.$confirm('确定要删除云菜谱吗？', ['取消', '确定'], function (value) {
                    if (value != '确定') {
                        return
                    }

                    that.$ajax('yunscreen_program_remove', { ids: ids }, function (response) {
                        var json = response || { code: -1 }
                        var pages = that.pages
                        if (json.code == 200) {
                            that.selected.map(function (item) {
                                pages.splice(pages.indexOf(item), 1)
                            })
                            that.selectFilter()
                            that.$toast('删除云菜谱')
                            that.nomore &&
                                that.$nextTick(function () {
                                    that.$refs.scroller.infiniteComplete()
                                })
                        } else {
                            that.$toast(json.msg || '删除云菜谱失败')
                        }
                    })
                })
            }
        },

        //预览
        toDetail: function (event, item) {
            var that = this
            var deviceId = this.$urlParam('deviceId') || ''

            if (!item) {
                that.$toast('未选中云菜谱')
            } else if (deviceId) {
                if (item.amountInfo.amount > 0) {
                    // 未支付的
                    if (window.plus) {
                        this.$toLink(
                            'yunscreen_detail.html',
                            {
                                programId: item.id,
                                deviceId: deviceId,
                            },
                            'none'
                        )

                        setTimeout(function () {
                            plus.webview.currentWebview().close('none', 300)
                        })
                    }

                    this.$toLink('user_order_maker.html', {
                        type: 'program',
                        ids: item.id,
                    })
                } else {
                    this.$toLink('yunscreen_detail.html', {
                        deviceId: deviceId,
                        programId: item.id,
                    })
                }
            } else if (this.selectable) {
                item.selected = !item.selected
                this.selectFilter()
            } else {
                event.stopPropagation()
                event.preventDefault()
                if (this.loading) {
                    return
                }
                this.loading = true
                this.$ajax('yunscreen_program_detail', { id: item.id }, function (response) {
                    var json = response || {}
                    var data = json.data || {}

                    if (json.code == 200) {
                        data.width = parseFloat(data.width)
                        data.height = parseFloat(data.height)
                        data.module = JSON.parse(data.data || '[]')

                        localStorage.setItem('maker_disabled', '[]')
                        localStorage.setItem('maker_page', JSON.stringify(data))
                        this.loading = false
                        this.$toLink('maker_editor.html', { type: 'yunscreen' })
                    } else {
                        this.loading = false
                        this.$toast(json.message || '获取云菜谱详情失败')
                    }
                })
            }
        },

        // 编辑
        toEdit: function (item) {
            if (this.loading) {
                return
            }

            if (this.selectable) {
                return
            }

            this.loading = true
            this.$ajax('yunscreen_program_detail', { id: item.id }, function (response) {
                var json = response || {}
                var data = json.data || {}

                if (json.code == 200) {
                    data.width = parseFloat(data.width)
                    data.height = parseFloat(data.height)
                    data.module = JSON.parse(data.data || '[]')

                    localStorage.setItem('maker_disabled', '[]')
                    localStorage.setItem('maker_page', JSON.stringify(data))
                    this.loading = false
                    this.$toLink('maker_editor.html', { type: 'yunscreen' })
                } else {
                    this.loading = false
                    this.$toast(json.message || '获取云菜谱详情失败')
                }
            })
        },

        // 去播放
        toPlay: function (item) {
            var deviceId = this.$urlParam('deviceId') || ''
            var config = JSON.parse(localStorage.getItem('config') || '{}')
            var deviceList = config.deviceList || []

            if (deviceId) {
                this.$toLink('yunscreen_detail.html', {
                    deviceId: deviceList[0].id,
                    programId: item.id,
                })
                setTimeout(function () {
                    plus.webview.currentWebview().close('none', 300)
                })
            } else if (deviceList.length == 0) {
                // 跳到扫码界面
                this.$toLink('index_scan.html', { fromPage: 'index.html', programId: item.id })
            } else {
                // 跳到列表
                this.$toLink('yunscreen_list.html', { programId: item.id })
            }
        },
        // 下单
        toOrder: function (id) {
            // 筛选出未支付的订单
            var unpay = this.selected.filter(function (item) {
                return (item.amountInfo || {}).amount > 0
            })
            var ids = id
                ? [id]
                : unpay.map(function (item) {
                      return item.id
                  })

            if (ids.length == 0) {
                if (unpay.length == 0) {
                    this.$toast('请选择云菜谱')
                } else {
                    this.$toast('您选中的云菜谱都已经支付')
                }
            } else {
                this.$toLink('user_order_maker.html', {
                    type: 'program',
                    ids: ids.join(','),
                })
            }
        },

        // 数据有更新
        makerComplete: function (page) {
            // 保存云菜谱单
            var that = this
            var thumb = {
                canvas: null,
                width: 500,
                height: (500 * page.height) / page.width,
            }
            var request = {}
            var notify = page.id ? '修改' : '添加'
            var item =
                page.id &&
                this.pages.filter(function (i) {
                    return i.id == page.id
                })[0] // 当前修改的块
            var gids = [] // 图片id
            var rids = [] // 素材图片 id
            var list = page.module || JSON.stringify(page.data || '[]')

            // 保存数据
            list.map(function (item) {
                if (/video|image/.test(item.type)) {
                    item.gid && gids.indexOf(item.gid) && gids.push(item.gid)
                    item.rid && rids.indexOf(item.rid) && rids.push(item.rid)
                }
            })

            request.id = page.id || ''
            request.templateId = page.templateId || 0 // 否    varchar 用户模板id，在未生成之前为空
            request.name = page.name || '' // 是    varchar 模板名称,建议使用图片名称
            request.useGallery = gids.join(',') // 是    varchar 使用图片id，例如：232,233
            request.useResources = rids.join(',') // 是   varchar 使用素材图片id，例如：232,233
            request.module = page.module || JSON.stringify(page.data || '[]')
            request.width = page.width || 1 // 是    varchar 数据（创建/编辑用）
            request.height = page.height || 1 // 是    varchar 数据（创建/编辑用）
            request.version = '2.0'

            // 如果没有id则先添加进去
            if (!item) {
                var width = (window.screen.width - 40) / 2 // 两个并一行
                item = JSON.parse(JSON.stringify(request))
                item.building = true
                item.thumbW = parseInt(item.width)
                item.thumbH = parseInt(item.height)
                item.paddingBottom = (100 * item.thumbH) / item.thumbW + '%'
                item.height = (width * item.thumbH) / item.thumbW + 60
                item.deviceList = []
                item.selected = false
                item.amountInfo = null

                this.pages.unshift(item)
            } else {
                item.building = true
                this.$set(this.pages, this.pages.indexOf(item), item)
            }

            this.$nextTick(function () {
                this.$refs.scroller.infiniteComplete()
            })
            this.$makerDrawPage(request, thumb, function (canvas) {
                if (!canvas) {
                    that.$toast('生成云菜谱失败，请重试')
                    item.building = false
                    that.pages.splice(that.pages.indexOf(item), 1)
                    that.$nextTick(function () {
                        that.$refs.scroller.infiniteComplete()
                    })
                } else {
                    request.data = JSON.stringify(request.module)
                    delete request.module
                    canvas.name = (request.name || '云菜谱缩略图') + '.png'

                    that.$uploadCanvas(canvas, function (imageUrl) {
                        if (!imageUrl) {
                            that.$toast('保存预览图失败，请重新制作')
                            item.building = false
                            that.pages.splice(that.pages.indexOf(item), 1)
                            that.$nextTick(function () {
                                that.$refs.scroller.infiniteComplete()
                            })
                        } else {
                            request.thumb = imageUrl // 是    varchar 预览图文件流（创建/编辑用）
                            that.$ajax('yunscreen_program_save', request, function (json) {
                                var data = (json || {}).data
                                if (json.code == 200) {
                                    that.$toast(notify + '云菜谱成功')
                                    Object.assign(item, data)
                                    item.building = false
                                    item.amountInfo = data.amountInfo || data.amount || {}
                                    that.$nextTick(function () {
                                        that.$refs.scroller.infiniteComplete()
                                    })
                                } else {
                                    that.$toast(notify + '云菜谱失败')
                                    item.building = false
                                    that.pages.splice(that.pages.indexOf(item), 1)
                                    that.$nextTick(function () {
                                        that.$refs.scroller.infiniteComplete()
                                    })
                                }
                            })
                        }
                    })
                }
            })
        },
    },
    created: function () {
        var that = this
        this.pagesUpdata(true, function () {
            // var temp = that.$urlParam('temp')
            // that.referer == 'maker_editor.html' && temp && that.selectTemp(temp);
            that.loading = false
        })

        // 来源是云菜谱，则生成一个云菜谱

        // 制作完成
        this.$on('global:maker_complete', that.makerComplete)

        // 选择完模板 生成图片并添加到云菜谱中
        // this.$on('global:yunprogram_list_temp', this.selectTemp)

        // 选择完图片
        // this.$on('global:yunprogram_list_picture', this.selectPicture)

        // 订单支付完
        this.$on('global:order_payfor_complete', function () {
            that.pagesUpdata(true)
        })
    },
})
